==> binary_codec-1.0.0.tar.gz: lib/src/binary_codec.dart <==
import 'dart:convert';
import 'dart:math';
import 'dart:typed_data';

final Codec<dynamic, Uint8List> binaryCodec = _BinaryCodec();

class _BinaryCodec extends Codec<dynamic, Uint8List> {
  Converter<Uint8List, dynamic> get decoder => _Decoder();
  Converter<dynamic, Uint8List> get encoder => _Encoder();
}

class _Encoder extends Converter<dynamic, Uint8List> {
  ByteData data;
  int offset;

  set next(int byte) {
    if (offset == data.lengthInBytes) {
      var newData = Uint8List(data.lengthInBytes * 2);
      newData.setRange(0, offset, data.buffer.asUint8List());
      data = newData.buffer.asByteData();
    }
    data.setUint8(offset++, byte);
  }

  Uint8List convert(dynamic input) {
    data = ByteData(8);
    offset = 0;
    encode(input);
    var result = Uint8List(offset);
    result.setRange(0, offset, data.buffer.asUint8List());
    data = null;
    return result;
  }

  void encode(dynamic value) {
    if (value == null)
      next = _Type.vNull;
    else if (value is bool)
      next = value ? _Type.vTrue : _Type.vFalse;
    else if (value is int) {
      if (value >= 0 && value < 128)
        next = _Type.vPosInt + value;
      else if (value < 0 && value > -64)
        next = _Type.vNegInt - value;
      else
        encodeInt(value);
    } else if (value is double)
      encodeDouble(value);
    else if (value is String)
      encodeBytes(utf8.encode(value), true);
    else if (value is Uint8List)
      encodeBytes(value, false);
    else if (value is List)
      encodeList(value);
    else if (value is Map)
      encodeMap(value);
    else {
      var message = 'Type ${value.runtimeType} not supported (value=$value)';
      throw ArgumentError(message);
    }
  }

  void encodeInt(int value) {
    int payLoadLength = ((value.bitLength + 1) / 8).ceil();
    next = _Type.vInt + payLoadLength - 1;
    if (value >= 0) {
      for (int i = 0; i < payLoadLength; i++) {
        next = value >> i * 8;
      }
    } else {
      value = ~value;
      for (int i = 0; i < payLoadLength; i++) {
        next = ~(value >> i * 8);
      }
    }
  }

  void encodeDouble(double value) {
    next = _Type.vDouble;
    var bytes = Uint8List(8);
    bytes.buffer.asByteData().setFloat64(0, value, Endian.little);
    for (var byte in bytes) next = byte;
  }

  void encodeBytes(Uint8List bytes, bool isText) {
    int length = bytes.length;
    if (length < 16) {
      next = (isText ? _Type.vShortText : _Type.vShortBlob) + length;
    } else if (length & (length - 1) == 0) {
      next =
          (isText ? _Type.vExpText : _Type.vExpBlob) + length.bitLength ~/ 16;
    } else {
      next = isText ? _Type.vText : _Type.vBlob;
      encodeInt(length);
    }
    growUntil(offset + length);
    data.buffer.asUint8List(offset).setRange(0, length, bytes);
    offset += length;
  }

  void growUntil(int capacity) {
    int newLength = data.lengthInBytes;
    while (newLength < offset + capacity) newLength *= 2;
    if (newLength != data.lengthInBytes) {
      var newData = ByteData(newLength);
      newData.buffer
          .asUint8List()
          .setRange(0, offset, data.buffer.asUint8List());
      data = newData;
    }
  }

  void encodeList(List value) {
    next = _Type.vList;
    encodeInt(value.length);
    for (var element in value) {
      encode(element);
    }
  }

  void encodeMap(Map value) {
    next = _Type.vMap;
    encodeInt(value.length);
    for (var entry in value.entries) {
      encode(entry.key);
      encode(entry.value);
    }
  }
}

class _Decoder extends Converter<Uint8List, dynamic> {
  ByteData data;
  int offset = 0;

  int get next => data.getUint8(offset++);

  convert(Uint8List input, [int offset]) {
    data = input.buffer.asByteData();
    offset = offset ?? 0;
    var value = decode();
    data = null;
    return value;
  }

  dynamic decode() {
    int type = next;
    if (type < _Type.vNegInt) {
      return type - _Type.vPosInt;
    } else if (type < _Type.vShortBlob) {
      return -(type - _Type.vNegInt);
    } else if (type < _Type.vShortText) {
      return decodeBlob(type - _Type.vShortBlob);
    } else if (type < _Type.vExpBlob) {
      return utf8.decode(decodeBlob(type - _Type.vShortText));
    } else if (type < _Type.vExpText) {
      return decodeBlob(pow(2, type - _Type.vExpBlob) * 16);
    } else if (type < _Type.vInt) {
      return utf8.decode(decodeBlob(pow(2, type - _Type.vExpText) * 16));
    } else if (type < _Type.vNull) {
      return decodeInt(type);
    } else if (type == _Type.vNull) {
      return null;
    } else if (type == _Type.vTrue) {
      return true;
    } else if (type == _Type.vFalse) {
      return false;
    } else if (type == _Type.vDouble) {
      return decodeDouble();
    } else if (type == _Type.vBlob) {
      return decodeBlob(decodeInt(next));
    } else if (type == _Type.vText) {
      return utf8.decode(decodeBlob(decodeInt(next)));
    } else if (type == _Type.vList) {
      return decodeList();
    } else if (type == _Type.vMap) {
      return decodeMap();
    } else {
      throw StateError('$type not handled');
    }
  }

  Uint8List decodeBlob(int length) {
    var list = Uint8List(length);
    list.setRange(0, length, data.buffer.asUint8List(), offset);
    offset += length;
    return list;
  }

  int decodeInt(int type) {
    var length = 1 + (type & 0x7);
    var value = 0;
    for (int i = 0; i < length; i++) {
      var byte = next;
      if (i == length - 1) {
        if (byte & 0x80 != 0) {
          value += pow(2, i * 8) * (byte & 0x7F);
          value = value - pow(2, length * 8 - 1);
          break;
        }
      }
      value += pow(2, i * 8) * byte;
    }
    return value;
  }

  double decodeDouble() {
    var list = Uint8List(8);
    list.setRange(0, 8, Iterable.generate(8, (_) => next));
    return list.buffer.asByteData().getFloat64(0, Endian.little);
  }

  List decodeList() {
    int length = decodeInt(next);
    var list = List();
    for (int i = 0; i < length; i++) {
      list.add(decode());
    }
    return list;
  }

  Map decodeMap() {
    int length = decodeInt(next);
    var map = Map();
    for (int i = 0; i < length; i++) {
      var key = decode();
      var value = decode();
      map[key] = value;
    }
    return map;
  }
}

class _Type {
  static const vPosInt = 0x00;
  static const vNegInt = 0x80;
  static const vShortBlob = 0xC0;
  static const vShortText = 0xD0;
  static const vExpBlob = 0xE0;
  static const vExpText = 0xE8;
  static const vInt = 0xF0;
  static const vNull = 0xF8;
  static const vTrue = 0xF9;
  static const vFalse = 0xFA;
  static const vDouble = 0xFB;
  static const vBlob = 0xFC;
  static const vText = 0xFD;
  static const vList = 0xFE;
  static const vMap = 0xFF;
}

==> binary_codec-1.0.3.tar.gz: lib/src/binary_codec.dart <==
import 'dart:convert';
import 'dart:math';
import 'dart:typed_data';

/// Codec that translate between dynamic and Uint8List
///
/// Supported types: Null, bool, int, double, String, Uint8List, List, Map
/// Maps can contain values and keys of any of the above listed types
/// Lists can contain values of any of the above listed types
final Codec<dynamic, Uint8List> binaryCodec = _BinaryCodec();

class _BinaryCodec extends Codec<dynamic, Uint8List> {
  /// Converts a Uint8List to the dynamic value that it represents
  ///
  /// May fail with an IndexError if the binary data is not formatted properly
  Converter<Uint8List, dynamic> get decoder => _Decoder();

  /// Converts a dynamic value to a representing byte-list
  ///
  /// May fail with an ArgumentError if the value's type isn't supported
  Converter<dynamic, Uint8List> get encoder => _Encoder();
}

class _Encoder extends Converter<dynamic, Uint8List> {
  ByteData data;
  int offset;

  set next(int byte) {
    if (offset == data.lengthInBytes) {
      var newData = Uint8List(data.lengthInBytes * 2);
      newData.setRange(0, offset, data.buffer.asUint8List());
      data = newData.buffer.asByteData();
    }
    data.setUint8(offset++, byte);
  }

  Uint8List convert(dynamic input) {
    data = ByteData(8);
    offset = 0;
    encode(input);
    var result = Uint8List(offset);
    result.setRange(0, offset, data.buffer.asUint8List());
    data = null;
    return result;
  }

  void encode(dynamic value) {
    if (value == null) {
      next = _Type.vNull;
    } else if (value is bool) {
      next = value ? _Type.vTrue : _Type.vFalse;
    } else if (value is int) {
      if (value >= 0 && value < 128) {
        next = _Type.vPosInt + value;
      } else if (value < 0 && value > -64) {
        next = _Type.vNegInt - value;
      } else {
        encodeInt(value);
      }
    } else if (value is double) {
      encodeDouble(value);
    } else if (value is String) {
      encodeBytes(utf8.encode(value), true);
    } else if (value is Uint8List) {
      encodeBytes(value, false);
    } else if (value is List) {
      encodeList(value);
    } else if (value is Map) {
      encodeMap(value);
    } else {
      var message = 'Type ${value.runtimeType} not supported (value=$value)';
      throw ArgumentError(message);
    }
  }

  void encodeInt(int value) {
    int payLoadLength = ((value.bitLength + 1) / 8).ceil();
    next = _Type.vInt + payLoadLength - 1;
    if (value >= 0) {
      for (int i = 0; i < payLoadLength; i++) {
        next = value >> i * 8;
      }
    } else {
      value = ~value;
      for (int i = 0; i < payLoadLength; i++) {
        next = ~(value >> i * 8);
      }
    }
  }

  void encodeDouble(double value) {
    next = _Type.vDouble;
    var bytes = Uint8List(8);
    bytes.buffer.asByteData().setFloat64(0, value, Endian.little);
    for (var byte in bytes) {
      next = byte;
    }
  }

  void encodeBytes(Uint8List bytes, bool isText) {
    int length = bytes.length;
    if (length < 16) {
      next = (isText ? _Type.vShortText : _Type.vShortBlob) + length;
    } else if (length & (length - 1) == 0) {
      next =
          (isText ? _Type.vExpText : _Type.vExpBlob) + length.bitLength ~/ 16;
    } else {
      next = isText ? _Type.vText : _Type.vBlob;
      encodeInt(length);
    }
    growUntil(offset + length);
    data.buffer.asUint8List(offset).setRange(0, length, bytes);
    offset += length;
  }

  void growUntil(int capacity) {
    int newLength = data.lengthInBytes;
    while (newLength < offset + capacity) {
      newLength *= 2;
    }
    if (newLength != data.lengthInBytes) {
      var newData = ByteData(newLength);
      newData.buffer
          .asUint8List()
          .setRange(0, offset, data.buffer.asUint8List());
      data = newData;
    }
  }

  void encodeList(List value) {
    next = _Type.vList;
    encodeInt(value.length);
    for (var element in value) {
      encode(element);
    }
  }

  void encodeMap(Map value) {
    next = _Type.vMap;
    encodeInt(value.length);
    for (var entry in value.entries) {
      encode(entry.key);
      encode(entry.value);
    }
  }
}

class _Decoder extends Converter<Uint8List, dynamic> {
  ByteData data;
  int offset = 0;

  int get next => data.getUint8(offset++);

  convert(Uint8List input, [int offset]) {
    data = input.buffer.asByteData();
    offset = offset ?? 0;
    var value = decode();
    data = null;
    return value;
  }

  dynamic decode() {
    int type = next;
    if (type < _Type.vNegInt) {
      return type - _Type.vPosInt;
    } else if (type < _Type.vShortBlob) {
      return -(type - _Type.vNegInt);
    } else if (type < _Type.vShortText) {
      return decodeBlob(type - _Type.vShortBlob);
    } else if (type < _Type.vExpBlob) {
      return utf8.decode(decodeBlob(type - _Type.vShortText));
    } else if (type < _Type.vExpText) {
      return decodeBlob(pow(2, type - _Type.vExpBlob) * 16);
    } else if (type < _Type.vInt) {
      return utf8.decode(decodeBlob(pow(2, type - _Type.vExpText) * 16));
    } else if (type < _Type.vNull) {
      return decodeInt(type);
    } else if (type == _Type.vNull) {
      return null;
    } else if (type == _Type.vTrue) {
      return true;
    } else if (type == _Type.vFalse) {
      return false;
    } else if (type == _Type.vDouble) {
      return decodeDouble();
    } else if (type == _Type.vBlob) {
      return decodeBlob(decodeInt(next));
    } else if (type == _Type.vText) {
      return utf8.decode(decodeBlob(decodeInt(next)));
    } else if (type == _Type.vList) {
      return decodeList();
    } else if (type == _Type.vMap) {
      return decodeMap();
    } else {
      throw StateError('$type not handled');
    }
  }

  Uint8List decodeBlob(int length) {
    var list = Uint8List(length);
    list.setRange(0, length, data.buffer.asUint8List(), offset);
    offset += length;
    return list;
  }

  int decodeInt(int type) {
    var length = 1 + (type & 0x7);
    var value = 0;
    for (int i = 0; i < length; i++) {
      var byte = next;
      if (i == length - 1) {
        if (byte & 0x80 != 0) {
          value += pow(2, i * 8) * (byte & 0x7F);
          value = value - pow(2, length * 8 - 1);
          break;
        }
      }
      value += pow(2, i * 8) * byte;
    }
    return value;
  }

  double decodeDouble() {
    var list = Uint8List(8);
    list.setRange(0, 8, Iterable.generate(8, (_) => next));
    return list.buffer.asByteData().getFloat64(0, Endian.little);
  }

  List decodeList() {
    int length = decodeInt(next);
    var list = List();
    for (int i = 0; i < length; i++) {
      list.add(decode());
    }
    return list;
  }

  Map decodeMap() {
    int length = decodeInt(next);
    var map = Map();
    for (int i = 0; i < length; i++) {
      var key = decode();
      var value = decode();
      map[key] = value;
    }
    return map;
  }
}

class _Type {
  static const vPosInt = 0x00;
  static const vNegInt = 0x80;
  static const vShortBlob = 0xC0;
  static const vShortText = 0xD0;
  static const vExpBlob = 0xE0;
  static const vExpText = 0xE8;
  static const vInt = 0xF0;
  static const vNull = 0xF8;
  static const vTrue = 0xF9;
  static const vFalse = 0xFA;
  static const vDouble = 0xFB;
  static const vBlob = 0xFC;
  static const vText = 0xFD;
  static const vList = 0xFE;
  static const vMap = 0xFF;
}
